@@ -30,6 +30,10 @@ protocol MediaMoreOptionsActionSheetDelegate {
     func mediaMoreOptionsActionSheetRemoveAddBookmarksView()
     func mediaMoreOptionsActionSheetDidToggleShuffle(_ mediaMoreOptionsActionSheet: MediaMoreOptionsActionSheet)
     func mediaMoreOptionsActionSheetDidTapRepeat(_ mediaMoreOptionsActionSheet: MediaMoreOptionsActionSheet)
+    func mediaMoreOptionsActionSheetPresentABRepeatView(with abView: ABRepeatView)
+    func mediaMoreOptionsActionSheetDidSelectAMark()
+    func mediaMoreOptionsActionSheetDidSelectBMark()
+    @objc optional func mediaMoreOptionsActionSheetShowPlaybackSpeedShortcut(_ displayView: Bool)
 }
 
 @objc (VLCMediaMoreOptionsActionSheet)
@@ -84,13 +88,13 @@ protocol MediaMoreOptionsActionSheetDelegate {
             playbackSpeedView.overrideUserInterfaceStyle = .dark
         }
         playbackSpeedView.delegate = self
+        playbackSpeedView.setupShortcutView()
         return playbackSpeedView
     }()
 
     private lazy var sleepTimerView: SleepTimerView = {
-        let sleepTimerView = Bundle.main.loadNibNamed("SleepTimerView",
-                                                      owner: nil,
-                                                      options: nil)?.first as! SleepTimerView
+        let nib = UINib(nibName: "SleepTimerView", bundle: nil)
+        let sleepTimerView = nib.instantiate(withOwner: nil, options: nil).first as! SleepTimerView
         sleepTimerView.frame = offScreenFrame
         if #available(iOS 13.0, *) {
             sleepTimerView.overrideUserInterfaceStyle = .dark
@@ -102,7 +106,7 @@ protocol MediaMoreOptionsActionSheetDelegate {
     private lazy var equalizerView: EqualizerView = {
         let equalizerView = EqualizerView()
         if #available(iOS 13.0, *) {
-            sleepTimerView.overrideUserInterfaceStyle = .dark
+            equalizerView.overrideUserInterfaceStyle = .dark
         }
 
         guard let playbackService = PlaybackService.sharedInstance() as? EqualizerViewDelegate else {
@@ -136,6 +140,12 @@ protocol MediaMoreOptionsActionSheetDelegate {
         return addBookmarksView
     }()
 
+    private lazy var abRepeatView: ABRepeatView = {
+        let abRepeatView = ABRepeatView(frame: CGRect(x: 0, y: 0, width: 500, height: 100))
+        abRepeatView.delegate = self
+        return abRepeatView
+    }()
+
     // MARK: - Initializers
     override init() {
         super.init()
@@ -226,7 +236,7 @@ protocol MediaMoreOptionsActionSheetDelegate {
         updateThemes()
     }
 
-    func addView(_ view: MediaPlayerActionSheetCellIdentifier) {
+    func addView(_ view: ActionSheetCellIdentifier) {
         switch view {
         case .filter:
             openOptionView(videoFiltersView)
@@ -280,6 +290,14 @@ extension MediaMoreOptionsActionSheet: PlaybackSpeedViewDelegate {
     func playbackSpeedViewHideIcon() {
         moreOptionsDelegate?.mediaMoreOptionsActionSheetHideIcon(for: .playbackSpeed)
     }
+
+    func playbackSpeedViewCanDisplayShortcutView() -> Bool {
+        return moreOptionsDelegate is AudioPlayerViewController
+    }
+
+    func playbackSpeedViewHandleShortcutSwitchChange(displayView: Bool) {
+        moreOptionsDelegate?.mediaMoreOptionsActionSheetShowPlaybackSpeedShortcut?(displayView)
+    }
 }
 
 // MARK: - SleepTimerViewDelegate
@@ -382,6 +400,25 @@ extension MediaMoreOptionsActionSheet: BookmarksViewDelegate {
     }
 }
 
+// MARK: - ABRepeatViewDelegate
+extension MediaMoreOptionsActionSheet: ABRepeatViewDelegate {
+    func abRepeatViewDidSelectAMark() {
+        moreOptionsDelegate?.mediaMoreOptionsActionSheetDidSelectAMark()
+    }
+
+    func abRepeatViewDidSelectBMark() {
+        moreOptionsDelegate?.mediaMoreOptionsActionSheetDidSelectBMark()
+    }
+
+    func abRepeatViewShowIcon(_ option: OptionsNavigationBarIdentifier) {
+        moreOptionsDelegate?.mediaMoreOptionsActionSheetShowIcon(for: option)
+    }
+
+    func abRepeatViewHideIcon(_ option: OptionsNavigationBarIdentifier) {
+        moreOptionsDelegate?.mediaMoreOptionsActionSheetHideIcon(for: option)
+    }
+}
+
 // MARK: - MediaPlayerActionSheetDelegate
 extension MediaMoreOptionsActionSheet: MediaPlayerActionSheetDelegate {
     func mediaPlayerActionSheetHeaderTitle() -> String? {
@@ -411,7 +448,7 @@ extension MediaMoreOptionsActionSheet: MediaPlayerActionSheetDelegate {
 // MARK: - MediaPlayerActionSheetDataSource
 extension MediaMoreOptionsActionSheet: MediaPlayerActionSheetDataSource {
 
-    private func selectViewToPresent(for cell: MediaPlayerActionSheetCellIdentifier) -> UIView {
+    private func selectViewToPresent(for cell: ActionSheetCellIdentifier) -> UIView {
         switch cell {
         case .filter:
             return videoFiltersView
@@ -425,6 +462,8 @@ extension MediaMoreOptionsActionSheet: MediaPlayerActionSheetDataSource {
             return chapterView
         case .bookmarks:
             return bookmarksView
+        case .abRepeat:
+            return abRepeatView
         default:
             return mockView
         }
@@ -434,13 +473,15 @@ extension MediaMoreOptionsActionSheet: MediaPlayerActionSheetDataSource {
         var models: [ActionSheetCellModel] = []
         let isAudioPlayer: Bool = moreOptionsDelegate is AudioPlayerViewController
 
-        MediaPlayerActionSheetCellIdentifier.allCases.forEach {
-            if $0 == .chapters && (currentMediaHasChapters == false || isAudioPlayer) {
+        ActionSheetCellIdentifier.allCases.forEach {
+            if $0 == .chapters && currentMediaHasChapters == false {
                 // Do not show the chapters category when there are no chapters.
                 return
             }
 
-            if $0 == .addBookmarks {
+            // Do not display these options in the action sheet.
+            if $0 == .addBookmarks || $0 == .blackBackground ||
+                $0 == .playNextItem || $0 == .playlistPlayNextItem {
                 return
             }
 
@@ -455,13 +496,12 @@ extension MediaMoreOptionsActionSheet: MediaPlayerActionSheetDataSource {
                 viewToPresent: selectViewToPresent(for: $0),
                 cellIdentifier: $0
             )
+
             if $0 == .interfaceLock {
                 cellModel.accessoryType = .toggleSwitch
                 cellModel.viewToPresent = nil
             } else if $0 == .equalizer {
                 cellModel.accessoryType = .popup
-            } else if $0 == .chapters {
-                cellModel.accessoryType = .popup
             } else if $0 == .repeatShuffle {
                 cellModel.accessoryType = .none
                 cellModel.viewToPresent = mockView
